@@ -81,6 +81,8 @@ if _enabled:
 
 
 class ContextPyopencl(XContext):
+    context_cache = {}
+
     @property
     def nplike_array_type(self):
         return cla.Array
@@ -128,20 +130,29 @@ class ContextPyopencl(XContext):
         super().__init__()
 
         # TODO assume one device only
-        if device is None:
+        if device in self.context_cache:
+            self.platform, self.device, self.context = self.context_cache[
+                device
+            ]
+        elif device is None:
             self.context = cl.create_some_context(interactive=False)
             self.device = self.context.devices[0]
             self.platform = self.device.platform
         else:
             if isinstance(device, str):
-                platform, device = map(int, device.split("."))
+                platform, device_ = map(int, device.split("."))
                 self.platform = cl.get_platforms()[platform]
-                self.device = self.platform.get_devices()[device]
+                self.device = self.platform.get_devices()[device_]
             else:
                 self.device = device
                 self.platform = device.platform
 
             self.context = cl.Context([self.device])
+            self.context_cache[device] = (
+                self.platform,
+                self.device,
+                self.context,
+            )
 
         self.queue = cl.CommandQueue(self.context)
 
@@ -253,28 +264,30 @@ class ContextPyopencl(XContext):
         device_id = self.platform.get_devices().index(self.device)
         return f"{type(self).__name__}:{platform_id}.{device_id}"
 
-    def nparray_to_context_array(self, arr):
-        """
-        Copies a numpy array to the device memory.
+    def nparray_to_context_array(self, arr, copy=False):
+        """Copies a numpy array to the device memory.
+
         Args:
             arr (numpy.ndarray): Array to be transferred
+            copy (bool): This parameter is ignored for OpenCL, as the data lives
+                on a different device.
 
         Returns:
             pyopencl.array.Array:The same array copied to the device.
-
         """
         dev_arr = cla.to_device(self.queue, arr)
         return dev_arr
 
-    def nparray_from_context_array(self, dev_arr):
-        """
-        Copies an array to the device to a numpy array.
+    def nparray_from_context_array(self, dev_arr, copy=False):
+        """Copies an array to the device to a numpy array.
 
         Args:
             dev_arr (pyopencl.array.Array): Array to be transferred.
+            copy (bool): This parameter is ignored for OpenCL, as the data lives
+                on a different device.
+
         Returns:
             numpy.ndarray: The same data copied to a numpy array.
-
         """
         return dev_arr.get()
 
